using System.Text;
using UnityEngine;
using UnityEditor;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.IO;
using System;


class CompObj
{
    //vars

    //runtime component we're attached to
    public Component comp;

    //the gameobject we're attached to
    public GameObject go;

    //int representing the PlaySaveComponent we're attached to.
    public int num_comp;

    //values of fileds and properties
    Dictionary<PropertyInfo, object> m_Values = new Dictionary<PropertyInfo, object>();
    Dictionary<FieldInfo, object> m_Values_fields = new Dictionary<FieldInfo, object>();
    
    //should the comp be saved?
    public bool save;

    //have we changed
    public bool changed = false;

    //current prop we're restoring, used to check Exception
    PropertyInfo cur_prop;

    List<string> ignored_properties = new List<string>();

    //constructor
    public CompObj(Component _comp, GameObject _go, int _num_comp)
    {
        comp = _comp;
        go = _go;

        save = false;
        num_comp = _num_comp;

        UpdateValuesFirst();
    }

    //update vars for first time (no check for changed)
    public void UpdateValuesFirst()
    {
        //properties
        List<PropertyInfo> properties;
        properties = GetProperties(comp);
        foreach (PropertyInfo property in properties)
        {
            m_Values[property] = (property.GetValue(comp, null));
        }

        //fields
        List<FieldInfo> fields;
        fields = GetFields(comp);
        foreach (FieldInfo field in fields)
        {
            m_Values_fields[field] = (field.GetValue(comp));
        }
    }

    //update vars, and check if they've changed
    public void UpdateValues()
    {
        //properties

        List<PropertyInfo> properties;
        properties = GetProperties(comp);
        foreach (PropertyInfo property in properties)
        {
            object old_value = m_Values[property];

            m_Values[property] = (property.GetValue(comp, null));

            if (!m_Values[property].Equals(old_value))
            {
                if (!changed)
                {
                    changed = true;

                    if (m_Values[property] is Array)
                    {
                        changed = false;

                        ArrayList old = new ArrayList((Array)old_value);
                        ArrayList cur = new ArrayList((Array)m_Values[property]); 

                        if (old.Count != cur.Count)
                            changed = true;
                        else
                        {
                            for (int i = 0; i < old.Count; i += 1)
                            {
                                if (old[i] != cur[i])
                                    changed = true;
                            }
                        }
                    }


                    if (m_Values[property] is ArrayList)
                    {
                        changed = false;

                        ArrayList old = (ArrayList)old_value;
                        ArrayList cur = (ArrayList)m_Values[property];

                        if (old.Count != cur.Count)
                            changed = true;
                        else
                        {
                            for (int i = 0; i < old.Count; i += 1)
                            {
                                if (old[i] != cur[i])
                                    changed = true;
                            }
                        }
                    }
               }
            }
        }

        //fields
        List<FieldInfo> fields;
        fields = GetFields(comp);
        foreach (FieldInfo field in fields)
        {
            object old_value = m_Values_fields[field];

            m_Values_fields[field] = (field.GetValue(comp));

            if (!m_Values_fields[field].Equals(old_value))
                changed = true;
        }
    }

    //return true if we're checking the given property.
    bool Exception(string comp_find, string prop_name_find)
    {
        if (comp.GetType().Name == comp_find && cur_prop.Name == prop_name_find)
            return true;
        return false;
    }

    //round the fields of a vector3 to 9 decimals, to prevent unity from putting 0.9999999 instead of 1
    Vector3 VectorRound(Vector3 ls)
    {
        ls.x = Mathf.Floor(ls.x * 1000000000f) / 1000000000f;
        ls.y = Mathf.Floor(ls.y * 1000000000f) / 1000000000f;
        ls.z = Mathf.Floor(ls.z * 1000000000f) / 1000000000f;//9 decimals should be plenty

        return ls;
    }

    //restore vars
    public void RestoreValues()
    {
        if (!save)
            return;

        //properties
        List<PropertyInfo> properties;
        properties = GetProperties(comp);

        foreach (PropertyInfo property in properties)
        {
            if (!property.CanWrite)
                continue;


            cur_prop = property;

            if (!m_Values.ContainsKey(property))
            {
                Debug.LogError("Unknown value: " + property.Name + ". Unable to restore this property. For more information read the readme.txt");
                continue;
            }

            //the following exceptions alter the stored value to fix some minor problems.

            //localScale of Transform needs to rounded
            if (Exception("Transform", "localScale"))
            {
                Vector3 ls = VectorRound((Vector3)m_Values[property]);
                m_Values[property] = ls;
            }

            //rotation needs to be rounded and doesn't always clip to 0 - 360
            if (Exception("Transform", "eulerAngles"))
            {
                Vector3 rs = (Vector3)m_Values[property];

                //clamp rotation to 0 - 360
                rs.x = (rs.x + 360) % 360;
                rs.y = (rs.y + 360) % 360;
                rs.z = (rs.z + 360) % 360;

                rs = VectorRound(rs);

                m_Values[property] = rs;
            }


            property.SetValue(comp, m_Values[property], null);
        }
        
        //fields
        List<FieldInfo> fields = GetFields(comp);;
        foreach (FieldInfo field in fields)
        {
            if (field.IsPrivate)
                continue;

            object val = m_Values_fields[field];
            field.SetValue(comp, val);
        }

        //inform unity the obj needs to be changed
        EditorUtility.SetDirty(comp);
    }

    void Except(string var)
    {
        ignored_properties.Add(var);
    }

    //list all properties.
    private List<PropertyInfo> GetProperties(Component component)
    {
        ignored_properties.Clear();
        ignored_properties = new List<string>(); ;
        List<PropertyInfo> properties = new List<PropertyInfo>();

        //ignore general properties.
        foreach (PropertyInfo property_info in typeof(Component).GetProperties())
        {
            ignored_properties.Add(property_info.Name);
        }

        //ignore the properties that can't be written.
        foreach (PropertyInfo property_info in component.GetType().GetProperties())
        {
            if (!property_info.CanWrite)
            {
                ignored_properties.Add(property_info.Name);
            }
        }

        //exceptions
        //some variables are instansiated when you acces them, like renderer.material
        //these need to be skipped, only renderer.sharedMaterial needs to be adjusted
        //this goes for many type of vars.
        switch (comp.GetType().Name)
        {
            //materials.
            case "MeshRenderer":
                Except("materials");
                Except("material");
                break;


            //mesh vs sharedMesh
            case "MeshFilter":
                Except("mesh");
                break;

            case "SkinnedMeshRenderer":
                Except("mesh");
                break;  

            //camera has some read-only properties, but that c# reports as writable.
            //pixelRect and projectionMatrix are set to wrong values, this is due to Unity.. :(
            case "Camera":
                Except("pixelHeight");
                Except("pixelWidth");
                Except("aspect");
                Except("pixelRect");
                Except("projectionMatrix");
                Except("worldToCameraMatrix");
                break;

            //rigidbody has a lot of vars that are just copies of Transform. This is to prevent Rigidbody from reporting it has changed while only transform was changed.
            case "Rigidbody":
                Except("position");
                Except("Bounds");
                Except("velocity");
                Except("angularVelocity");
                Except("rotation");
                break;


            //colliders, material vs sharedMaterial and also some vars that are just copies.
            case "BoxCollider":
                Except("material");
                break;

            case "SphereCollider":
                Except("material");
                break;

            case "CapsuleCollider":
                Except("material");
                break;

            case "MeshCollider":
                Except("material");
                break;

            case "WheelCollider":
                Except("material");
                break;

            case "AudioSource":
                Except("minVolume");
                Except("maxVolume");
                Except("rolloffFactor");
                Except("minVolume");
                Except("minVolume");
                break;
        }

        foreach (PropertyInfo property_info in component.GetType().GetProperties())
        {
            if (ignored_properties.Contains(property_info.Name))
                continue;

            properties.Add(property_info);
        }

        return properties;
    }

    //list all fields.
    private List<FieldInfo> GetFields(Component component)
    {
        List<string> ignored_properties = new List<string>(); ;
        List<FieldInfo> properties = new List<FieldInfo>();

        //ignore general fields.
        foreach (FieldInfo property_info in typeof(Component).GetFields())
        {
            ignored_properties.Add(property_info.Name);
        }

        foreach (FieldInfo property_info in component.GetType().GetFields())
        {
            if (ignored_properties.Contains(property_info.Name) || property_info.IsPrivate)
                continue;

            properties.Add(property_info);
        }

        return properties;
    }
};





[CustomEditor(typeof(PlaySaveComponent))]
class PlaySave : Editor
{

    //editor callbacks.
    private static EditorApplication.CallbackFunction update_func;

    //component managment.
    private static List<CompObj> components = new List<CompObj>();
    private static List<Component> components_got = new List<Component>();
    
    //current attached component
    PlaySaveComponent _target;
    PlaySaveComponent prev_target;

    static bool prev_play = false ;
    static bool quit = false;

    bool first_frame = false;


    public void Awake()
    {
        _target = target as PlaySaveComponent;

        update_func = new EditorApplication.CallbackFunction(Update);
        EditorApplication.update += update_func;
    }



    //update: Track components
    public void Update()
    {
        _target = target as PlaySaveComponent;

        if (!prev_play && EditorApplication.isPlaying)
        {
            components.Clear();
            components_got.Clear();
        }

        if (prev_play && !EditorApplication.isPlayingOrWillChangePlaymode)          
            quit = true;

        if (!EditorApplication.isPlaying && quit)
        {
            quit = false;
            OnApplicationQuit();
        }

        prev_play = EditorApplication.isPlaying;

        if (EditorApplication.isPlaying)
        {
            if (_target == null)
                return;

            //update vars and components
            if (_target != prev_target)
                first_frame = true;

            Component[] obj_comps = _target.GetComponents(typeof(Component));

            //Get components.
            //loop through them and check if they need to be added to the list.
            for (int i = 0; i < obj_comps.Length; i += 1)
            {
                if (!components_got.Contains(obj_comps[i]) && !_target.added_runtime)
                {
                    if (obj_comps[i] != _target)
                    {
                        CompObj c = new CompObj(obj_comps[i], _target.gameObject, _target.num_comp);

                        if (_target.always_save)
                            c.save = true;

                        components.Add(c);
                        components_got.Add(obj_comps[i]);

                        if (!first_frame)
                        {
                            c.changed = true;
                        }
                    }
                }
            }

            first_frame = false;


            //update the stored values.
            for (int i = 0; i < components.Count; i += 1)
            {
                if (components[i].comp != null)
                    components[i].UpdateValues();
                else
                {
                    components.Remove(components[i]);
                    i -= 1;
                }
            }           
        }
        else
        {
            PlaySaveComponent[] save_comps = (PlaySaveComponent[])FindObjectsOfType(typeof(PlaySaveComponent));

            //update num_comp of the PlaySaveComponents
            for (int i = 0; i < save_comps.Length; i += 1)
                save_comps[i].num_comp = i;
        }

        prev_target = _target;
    }



    //onGui:Update the GUI
    public override void OnInspectorGUI()
    {
        EditorGUIUtility.LookLikeControls(140.0f, 100.0f);//make the toggle a bit larger.

        //draw
        if (components.Count == 0 && !_target.added_runtime){
            _target.always_save = EditorGUILayout.Toggle("Always save changes", _target.always_save);
            return;
        }


        int change = 0;
        int save = 0;
        for (int i = 0; i < components.Count; i += 1)
        {
            if (components[i].go != _target.gameObject)
                continue;

            if (components[i].changed)
                change += 1;
            if (components[i].save)
                save += 1;
        }


        if (change == 0 && !_target.added_runtime)
            return;

        if (EditorApplication.isPlaying)
        {
            EditorGUILayout.BeginVertical("Button");

            //if we're added at runtime, show a message that this isn't gonna work.
            if (_target.added_runtime)
            {
                GUIStyle style = new GUIStyle();
                style.wordWrap = true;
                EditorGUIUtility.LookLikeControls(Screen.width, Screen.width);
                EditorGUILayout.TextArea("Cannot save because when PlaySave is added while playing. For more information see the readme", style);
                EditorGUIUtility.LookLikeControls();
                EditorGUILayout.EndVertical();
                return;
            }

            if (change > 1)
            {
                if (save == components.Count)
                {
                    if (GUILayout.Button("Don't save"))
                        for (int i = 0; i < components.Count; i += 1)
                            components[i].save = false;
                }
                else
                {
                    if (GUILayout.Button("Save all"))
                        for (int i = 0; i < components.Count; i += 1)
                            components[i].save = true;
                }
            }

            EditorGUILayout.Space();


            for (int i = 0; i < components.Count; i += 1)
            {
                CompObj c = components[i];
                if (c.go == _target.gameObject && c.changed)
                    c.save = EditorGUILayout.Toggle(c.comp.GetType().Name, c.save);
            }

            EditorGUILayout.EndVertical();
        }
        else
        {
            _target.always_save = EditorGUILayout.Toggle("Always save changes", _target.always_save);

            if (_target.dont_show_restore)
                return;

            EditorGUILayout.BeginVertical("Button");

            if (change > 1 && GUILayout.Button("Discard all changes"))
            {
                components.Clear();
            }

            for (int i = 0; i < components.Count; i += 1)
            {
                CompObj c = components[i];

                if (c.comp == null)
                {
                    components.Remove(c);
                    i -= 1;
                }


                if (!c.changed || c.go != _target.gameObject)
                    continue;



                EditorGUILayout.BeginHorizontal();
                EditorGUILayout.PrefixLabel(c.comp.GetType().Name);
                EditorGUILayout.Space();

                if (GUILayout.Button("Restore"))
                {
                    Undo.RegisterUndo(c.comp, "PlaySave restore " + c.comp.GetType().Name);

                    c.save = true;
                    c.RestoreValues();

                    components.Remove(c);
                }

                EditorGUILayout.Space();

                if (GUILayout.Button("Discard"))
                {
                    components.Remove(c);
                    i -= 1;
                }

                EditorGUILayout.EndHorizontal();

                //if (c.go == _target.gameObject && c.changed)
                    //c.save = EditorGUILayout.Toggle(c.comp.GetType().Name, c.save);
            }

            EditorGUILayout.EndVertical();
        }

        EditorGUIUtility.LookLikeControls();
    }



    //delegate for clearing the list
    static private bool RemoveComp(CompObj c)
    {
        if (!c.save && c.changed)
            return false; 

        return true;
    }

    void DeleteRuntimes()
    {
        for (int i = 0; i < components.Count; i += 1)
        {
            if (components[i].go.GetComponent<PlaySaveComponent>() == null)
            {
                components.Remove(components[i]);
                i -= 1;
            }
        }
    }

    //OnApplicationQuit: Restore the checked vars.
    public void OnApplicationQuit()
    {
       
        object ps_objs = FindObjectsOfType(typeof(PlaySaveComponent));
        PlaySaveComponent[] save_comps = (PlaySaveComponent[])ps_objs;

        //register undo state

        //check if any component will be restored
        bool change = false;
        for (int i = 0; i < components.Count; i += 1)
            if (components[i].save)
            {
                change = true;
                break;
            }

        //if so, make a undo restore point
        if (change)
        {
            for (int i = 0; i < save_comps.Length; i++)
            {
                Component[] comps = save_comps[i].GetComponents<Component>();

                for (int j = 0; j < comps.Length; j++)
                {
                    Undo.RegisterUndo(comps[j], "PlaySave");
                }
            }
        }

        _target = target as PlaySaveComponent;

        //transfer components 
        for (int i = 0; i < components.Count; i += 1)
        {
            bool found = false;

            if (components[i].changed)
            {
                foreach (PlaySaveComponent save_comp in save_comps)
                {
                    if (save_comp.num_comp == components[i].num_comp)
                    {
                        found = true;

                        Component c = save_comp.GetComponent(components[i].comp.GetType());

                        if (c == null)
                        {
                            if (components[i].save)
                                c = save_comp.gameObject.AddComponent(components[i].comp.GetType().Name);
                            else
                            {
                                components.RemoveAt(i);
                                i -= 1;
                            }
                        }

                        components[i].comp = save_comp.GetComponent(components[i].comp.GetType());
                        break;
                    }
                }
                
                if (!found)
                {
                    components.RemoveAt(i);
                    i -= 1;
                }
            }

        }


        //restore the values
        for (int i = 0; i < components.Count; i += 1)
            components[i].RestoreValues();


        //delete all components that haven't been changed or saved/
        Predicate<CompObj> remove = new Predicate<CompObj>(RemoveComp);
        components.RemoveAll(remove);

        //clear the components_got
        components_got.Clear();
    }
}


