////////////////////////////////////////////////////////////////////////
// PieMenuCenter.cs
// By Don Hopkins (dhopkins@DonHopkins.com)
//
// Copyright 2012 Don Hopkins.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


////////////////////////////////////////////////////////////////////////
// Demo of how to make pie menu feedback like the head in the middle
// of The Sims pie menus, but with YOUR head from the webcam!


////////////////////////////////////////////////////////////////////////


using UnityEngine;
using System.Collections;


////////////////////////////////////////////////////////////////////////


public class PieMenuCenter : MonoBehaviour {


    ////////////////////////////////////////////////////////////////////////
    // Static methods.


    public static PieMenuCenter GetCenter(Transform transform)
    {
        while (transform != null) {
            PieMenuCenter center =
                transform.gameObject.GetComponent<PieMenuCenter>();

            if (center != null) {
                return center;
            }

            transform =
                transform.parent;
        }

        return null;
    }


    ////////////////////////////////////////////////////////////////////////
    // Instance methods.


    ////////////////////////////////////////////////////////////////////////
    // Parents.


	/// <summary>
	/// Gets the PieMenu of this PieMenuCenter.
	/// </summary>
    public PieMenu GetPieMenu()
    {
        return PieMenu.GetPieMenu(transform);
    }


	/// <summary>
	/// Gets the PieMenuRoot of this PieMenuCenter.
	/// </summary>
    public PieMenuRoot GetRoot()
    {
        return PieMenuRoot.GetRoot();
    }


	/// <summary>
	/// Gathers the game objects.
	/// </summary>
    public void GatherGameObjects(Transform parentTransform)
    {
        gameObject.transform.parent =
            parentTransform;

        foreach (Transform child in gameObject.transform) {
            PieMenuTracker tracker = child.GetComponent<PieMenuTracker>();
            if (tracker != null) {
                tracker.GatherGameObjects(parentTransform);
            }
        }
    }


	/// <summary>
	/// Draw the center.
	/// </summary>
    public void Draw(bool drawGUI)
    {
        PieMenu pieMenu =
            GetPieMenu();
        PieMenuRoot root =
            GetRoot();

        gameObject.layer =
            root.objectLayer;

        Vector3 screenPosition =
            new Vector3(
                pieMenu.centerX, 
                pieMenu.centerY, 
                root.pieMenuCamera.nearClipPlane);

        Ray ray =
            root.pieMenuCamera.ScreenPointToRay(
                screenPosition);

        Debug.DrawRay(ray.origin, ray.direction * 10.0f, Color.blue);

        Vector3 worldPosition =
            ray.GetPoint(
                root.objectDistance);

        float scale =
            pieMenu.centerScale * 
            pieMenu.objectScale * 
            root.itemObjectScale * 
            root.itemObjectCameraScale;

        gameObject.transform.position =
            worldPosition;
        gameObject.transform.localScale =
            new Vector3(scale, scale, scale);

        foreach (Transform child in gameObject.transform) {
            PieMenuTracker tracker = child.GetComponent<PieMenuTracker>();
            if (tracker != null) {
                tracker.Draw(drawGUI);
            }
        }

    }


}


////////////////////////////////////////////////////////////////////////
